@@ -15,65 +15,27 @@ import stat
 import struct
 import sys
 import sysconfig
+import tempfile
 import time
 from typing import Any, Dict, List, Optional
+import uuid
 import zipfile
 
-import cx_Freeze
-from cx_Freeze.darwintools import DarwinFile, MachOReference, DarwinFileTracker
+from .common import (
+    ConfigError,
+    get_resource_file_path,
+    process_path_specs,
+    validate_args,
+)
+from .darwintools import DarwinFile, MachOReference, DarwinFileTracker
+from .finder import ModuleFinder
+
+if sys.platform == "win32":
+    import cx_Freeze.util
 
 __all__ = ["ConfigError", "ConstantsModule", "Executable", "Freezer"]
 
 
-def process_path_specs(specs):
-    """Prepare paths specified as config.
-
-    The input is a list of either strings, or 2-tuples (source, target).
-    Where single strings are supplied, the basenames are used as targets.
-    Where targets are given explicitly, they must not be absolute paths.
-
-    Returns a list of 2-tuples, or throws ConfigError if something is wrong
-    in the input.
-    """
-    processedSpecs = []
-    for spec in specs:
-        if not isinstance(spec, (list, tuple)):
-            source = spec
-            target = None
-        elif len(spec) != 2:
-            raise ConfigError(
-                "path spec must be a list or tuple of length two"
-            )
-        else:
-            source, target = spec
-        source = os.path.normpath(source)
-        if not target:
-            target = os.path.basename(source)
-        elif os.path.isabs(target):
-            raise ConfigError(
-                "target path for include file may not be an absolute path"
-            )
-        processedSpecs.append((source, target))
-    return processedSpecs
-
-
-def get_resource_file_path(dirName, name, ext):
-    """Return the path to a resource file shipped with cx_Freeze.
-
-    This is used to find our base executables and initscripts when they are
-    just specified by name.
-    """
-    if os.path.isabs(name):
-        return name
-    name = os.path.normcase(name)
-    fullDir = os.path.join(os.path.dirname(cx_Freeze.__file__), dirName)
-    if os.path.isdir(fullDir):
-        for fileName in os.listdir(fullDir):
-            checkName, checkExt = os.path.splitext(os.path.normcase(fileName))
-            if name == checkName and ext == checkExt:
-                return os.path.join(fullDir, fileName)
-
-
 class Freezer:
     def __init__(
         self,
@@ -140,7 +102,7 @@ class Freezer:
             print(warning_msg)
             print("version must be specified")
             return
-        fileName = exe.targetName
+        fileName = os.path.join(self.targetDir, exe.target_name)
         versionInfo = VersionInfo(
             self.metadata.version,
             comments=self.metadata.long_description,
@@ -262,8 +224,8 @@ class Freezer:
 
     def _FreezeExecutable(self, exe):
         finder = self.finder
-        finder.IncludeFile(exe.script, exe.moduleName)
-        finder.IncludeFile(exe.initScript, exe.initModuleName)
+        finder.IncludeFile(exe.main_script, exe.main_module_name)
+        finder.IncludeFile(exe.init_script, exe.init_module_name)
         startupModule = get_resource_file_path(
             "initscripts", "__startup__", ".py"
         )
@@ -291,10 +253,10 @@ class Freezer:
         # Copy the python dynamic libraries and the frozen executable
         if sys.platform == "win32":
             # Copy the python dynamic libraries into build root folder
-            target_dir = os.path.dirname(exe.targetName)
+            target_dir = self.targetDir
         else:
             # Always copy the python dynamic libraries into lib folder
-            target_dir = os.path.join(os.path.dirname(exe.targetName), "lib")
+            target_dir = os.path.join(self.targetDir, "lib")
         for source in dependent_files:
             target = os.path.join(target_dir, os.path.basename(source))
             if sys.platform == "darwin":
@@ -311,15 +273,16 @@ class Freezer:
                 self._CopyFile(
                     source, target, copyDependentFiles=True, includeMode=True
                 )
+        target_path = os.path.join(self.targetDir, exe.target_name)
         self._CopyFile(
             exe.base,
-            exe.targetName,
+            target_path,
             copyDependentFiles=False,
             includeMode=True,
         )
-        if not os.access(exe.targetName, os.W_OK):
-            mode = os.stat(exe.targetName).st_mode
-            os.chmod(exe.targetName, mode | stat.S_IWUSR)
+        if not os.access(target_path, os.W_OK):
+            mode = os.stat(target_path).st_mode
+            os.chmod(target_path, mode | stat.S_IWUSR)
 
         if self.includeMSVCR:
             self._IncludeMSVCR(exe)
@@ -327,14 +290,27 @@ class Freezer:
         # Copy icon
         if exe.icon is not None:
             if sys.platform == "win32":
-                import cx_Freeze.util
-
-                cx_Freeze.util.AddIcon(exe.targetName, exe.icon)
+                try:
+                    cx_Freeze.util.AddIcon(target_path, exe.icon)
+                except RuntimeError as exc:
+                    print("*** WARNING ***", exc)
+                except OSError as exc:
+                    if "\\WindowsApps\\" in sys.base_prefix:
+                        print(
+                            "*** WARNING *** Because of restrictions on "
+                            "Microsoft Store apps, Python scripts may not "
+                            "have full write access to built executable.\n"
+                            "You will need to install the full installer.\n"
+                            "The following error was returned:"
+                        )
+                        print(exc)
+                    else:
+                        raise
             else:
-                targetName = os.path.join(
-                    os.path.dirname(exe.targetName), os.path.basename(exe.icon)
+                target_icon = os.path.join(
+                    self.targetDir, os.path.basename(exe.icon)
                 )
-                self._CopyFile(exe.icon, targetName, copyDependentFiles=False)
+                self._CopyFile(exe.icon, target_icon, copyDependentFiles=False)
 
         if self.metadata is not None and sys.platform == "win32":
             self._AddVersionResource(exe)
@@ -375,8 +351,6 @@ class Freezer:
         be included, generally because they contain standard system
         libraries."""
         if sys.platform == "win32":
-            import cx_Freeze.util
-
             systemDir = cx_Freeze.util.GetSystemDir()
             windowsDir = cx_Freeze.util.GetWindowsDir()
             return [windowsDir, systemDir, os.path.join(windowsDir, "WinSxS")]
@@ -468,7 +442,7 @@ class Freezer:
     def _GetModuleFinder(self, argsSource=None):
         if argsSource is None:
             argsSource = self
-        finder = cx_Freeze.ModuleFinder(
+        finder = ModuleFinder(
             self.includeFiles,
             self.excludes,
             self.path,
@@ -487,7 +461,7 @@ class Freezer:
         return finder
 
     def _IncludeMSVCR(self, exe):
-        targetDir = os.path.dirname(exe.targetName)
+        targetDir = self.targetDir
         for fullName in self.files_copied:
             path, name = os.path.split(os.path.normcase(fullName))
             if name.startswith("msvcr") and name.endswith(".dll"):
@@ -495,9 +469,9 @@ class Freezer:
                     sourceName = os.path.join(self.msvcRuntimeDir, otherName)
                     if not os.path.exists(sourceName):
                         continue
-                    targetName = os.path.join(targetDir, otherName)
+                    target_name = os.path.join(targetDir, otherName)
                     self._CopyFile(
-                        sourceName, targetName, copyDependentFiles=False
+                        sourceName, target_name, copyDependentFiles=False
                     )
                 break
 
@@ -609,9 +583,6 @@ class Freezer:
                     "excluded from zip file".format(name)
                 )
 
-        for executable in self.executables:
-            executable._VerifyConfiguration(self)
-
     def _WriteModules(self, fileName, finder):
         self.constantsModule.Create(finder)
         modules = [
@@ -704,18 +675,18 @@ class Freezer:
                 if module.code is None:
                     parts.pop()
                     parts.append(os.path.basename(module.file))
-                    targetName = os.path.join(targetDir, *parts)
+                    target_name = os.path.join(targetDir, *parts)
                     self._CopyFile(
                         module.file,
-                        targetName,
+                        target_name,
                         copyDependentFiles=True,
                         relativeSource=True,
                     )
                 else:
                     if module.path is not None:
                         parts.append("__init__")
-                    targetName = os.path.join(targetDir, *parts) + ".pyc"
-                    with open(targetName, "wb") as fp:
+                    target_name = os.path.join(targetDir, *parts) + ".pyc"
+                    with open(target_name, "wb") as fp:
                         fp.write(data)
 
             # otherwise, write to the zip file
@@ -825,67 +796,113 @@ class Freezer:
         return
 
 
-class ConfigError(Exception):
-    def __init__(self, msg):
-        self.what = msg
-        super().__init__()
-
-    def __str__(self):
-        return self.what
-
-
 class Executable:
+
+    _base: str
+    _init_script: str
+    _internal_name: str
+    _name: str
+    _ext: str
+
     def __init__(
         self,
-        script,
-        initScript=None,
-        base=None,
-        targetName=None,
-        icon=None,
-        shortcutName=None,
-        shortcutDir=None,
-        copyright=None,
-        trademarks=None,
+        script: str,
+        init_script: Optional[str] = None,
+        base: Optional[str] = None,
+        target_name: Optional[str] = None,
+        icon: Optional[str] = None,
+        shortcut_name: Optional[str] = None,
+        shortcut_dir: Optional[str] = None,
+        copyright: Optional[str] = None,
+        trademarks: Optional[str] = None,
+        *,
+        initScript: Optional[str] = None,
+        targetName: Optional[str] = None,
+        shortcutName: Optional[str] = None,
+        shortcutDir: Optional[str] = None,
     ):
-        self.script = script
-        self.initScript = initScript or "Console"
-        self.base = base or "Console"
-        self.targetName = targetName
+        self.main_script: str = script
+        self.init_script = validate_args(
+            "init_script", init_script, initScript
+        )
+        self.base = base
+        self.target_name = validate_args(
+            "target_name", target_name, targetName
+        )
         self.icon = icon
-        self.shortcutName = shortcutName
-        self.shortcutDir = shortcutDir
+        self.shortcut_name = validate_args(
+            "shortcut_name", shortcut_name, shortcutName
+        )
+        self.shortcut_dir = validate_args(
+            "shortcut_dir", shortcut_dir, shortcutDir
+        )
         self.copyright = copyright
         self.trademarks = trademarks
 
     def __repr__(self):
-        return "<Executable script=%s>" % self.script
+        return f"<Executable script={self.main_script}>"
 
-    def _VerifyConfiguration(self, freezer):
-        self._GetInitScriptFileName()
-        self._GetBaseFileName()
-        if self.targetName is None:
-            name, _ = os.path.splitext(os.path.basename(self.script))
-            ext = os.path.splitext(self.base)[1]
-            self.targetName = name + ext
-        name, ext = os.path.splitext(self.targetName)
-        if ext == "" and sys.platform == "win32":
-            self.targetName += ".exe"
-        self.moduleName = "%s__main__" % os.path.normcase(name)
-        self.initModuleName = "%s__init__" % os.path.normcase(name)
-        self.targetName = os.path.join(freezer.targetDir, self.targetName)
+    @property
+    def base(self) -> str:
+        return self._base
 
-    def _GetBaseFileName(self):
-        name = self.base
+    @base.setter
+    def base(self, name: Optional[str]):
+        name = name or "Console"
         ext = ".exe" if sys.platform == "win32" else ""
-        self.base = get_resource_file_path("bases", name, ext)
-        if self.base is None:
+        self._base = get_resource_file_path("bases", name, ext)
+        if self._base is None:
             raise ConfigError(f"no base named {name}")
 
-    def _GetInitScriptFileName(self):
-        name = self.initScript
-        self.initScript = get_resource_file_path("initscripts", name, ".py")
-        if self.initScript is None:
-            raise ConfigError(f"no initscript named {name}")
+    @property
+    def init_module_name(self) -> str:
+        return f"{self._internal_name}__init__"
+
+    @property
+    def init_script(self) -> str:
+        return self._init_script
+
+    @init_script.setter
+    def init_script(self, name: Optional[str]):
+        name = name or "Console"
+        self._init_script = get_resource_file_path("initscripts", name, ".py")
+        if self._init_script is None:
+            raise ConfigError(f"no init_script named {name}")
+
+    @property
+    def main_module_name(self) -> str:
+        return f"{self._internal_name}__main__"
+
+    @property
+    def target_name(self) -> str:
+        return self._name + self._ext
+
+    @target_name.setter
+    def target_name(self, name: Optional[str]):
+        if name is None:
+            name = os.path.splitext(os.path.basename(self.main_script))[0]
+            ext = os.path.splitext(self.base)[1]
+        else:
+            if name != os.path.basename(name):
+                raise ConfigError(
+                    "target_name should only be the name, for example: "
+                    f"{os.path.basename(name)}"
+                )
+            if sys.platform == "win32":
+                if name.endswith(".exe"):
+                    name, ext = os.path.splitext(name)
+                else:
+                    ext = ".exe"
+            else:
+                ext = ""
+        self._name = name
+        self._ext = ext
+        name = os.path.normcase(name).replace(" ", "_")
+        name = name.partition("-")[0]
+        name = name.partition(".")[0]
+        if not name.isidentifier():
+            raise ConfigError(f"Invalid name for target_name ({self._name!r})")
+        self._internal_name = name
 
 
 class ConstantsModule:
@@ -937,15 +954,17 @@ class ConstantsModule:
         self.values["BUILD_TIMESTAMP"] = today.strftime(self.time_format)
         self.values["BUILD_HOST"] = socket.gethostname().split(".")[0]
         self.values["SOURCE_TIMESTAMP"] = stamp.strftime(self.time_format)
-        module = finder._AddModule(self.module_name)
         source_parts = []
         names = list(self.values.keys())
         names.sort()
         for name in names:
             value = self.values[name]
             source_parts.append(f"{name} = {value!r}")
-        source = "\n".join(source_parts)
-        module.code = compile(source, "%s.py" % self.module_name, "exec")
+        filename = os.path.join(tempfile.gettempdir(), f"{uuid.uuid4()}.py")
+        with open(filename, "w") as fp:
+            fp.write("\n".join(source_parts))
+        module = finder.IncludeFile(filename, self.module_name)
+        os.remove(filename)
         return module
 
 
